#![allow(dead_code)]
use super::challenge::NodeUpdate;
use crate::datastore::Store;
use crate::grpc::challenge::update_client::UpdateClient;
use std::sync::Arc;
use std::thread::JoinHandle;
use tokio::sync::broadcast::Sender;
use tokio::task::JoinSet;
use tokio_stream::wrappers::BroadcastStream;
use tokio_stream::StreamExt;

struct Connection {
    ds: Arc<Store>,
    tx: Sender<NodeUpdate>,
}

#[derive(Clone)]
struct ConnManager {
    ds: Arc<Store>,
    tx: Sender<NodeUpdate>,
}

impl ConnManager {
    fn init(ds: Arc<Store>, tx: Sender<NodeUpdate>) -> Self {
        Self { ds, tx }
    }

    async fn connect(self, node_ip: String) {
        let mut client = UpdateClient::connect(format!("http://{node_ip}:50051"))
            .await
            .unwrap();

        let rx = self.tx.subscribe();
        let rx_stream = BroadcastStream::new(rx).filter_map(|n| n.ok());
        let response = client.get_updates(rx_stream).await.unwrap();
        let mut resp_stream = response.into_inner();

        while let Some(update) = resp_stream.message().await.unwrap() {
            if self.ds.process_grpc_update(update.clone()).await {
                if let Err(_) = self.tx.send(update.clone()) {
                    println!("tokio broadcast receivers had an issue consuming the channel");
                }
            };
        }

    }
}

fn init_connections(ds: Arc<Store>, tx: Sender<NodeUpdate>) -> JoinHandle<()> {
    std::thread::spawn(move || {
        tokio::runtime::Runtime::new().unwrap().block_on(async {
            let mut only_online_nodes = true;
            loop {
                let mut set = JoinSet::new();
                let nodes = ds.get_random_nodes(only_online_nodes).await;
                for node in nodes {
                    let conn = ConnManager::init(ds.clone(), tx.clone());
                    set.spawn(conn.connect(node));
                }
                while let Some(_) = set.join_next().await {}
                only_online_nodes = !only_online_nodes;
            }
        });
    })
}
